;;; ~/.doom.d/+ui.el -*- lexical-binding: t; -*-
;; Appearance settings
;; Set Font and theme
(setq doom-font (font-spec :family "Iosevka Medium" :size 26)
      doom-theme 'doom-one)
;; Display Magit status in fullscreen
(after! magit
  (setq magit-display-buffer-function #'magit-display-buffer-fullframe-status-v1))
;; Match pdf-tools colors to the theme and fit to page by default
(after! pdf-tools
  (setq pdf-view-midnight-colors '("#BBC2CF" . "#282C34"))
  (add-hook 'pdf-tools-enabled-hook 'pdf-view-midnight-minor-mode))
;; Customize modeline
(setq doom-modeline-major-mode-icon t
      doom-modeline-buffer-encoding nil)
;; Hide direnv environment variables popup
(setq direnv-always-show-summary nil)
;; Change splash-screen banner
(setq fancy-splash-image "~/.doom.d/assets/emacs.svg")
;; Disable LSP documentation popup
(after! lsp-ui
  (setq lsp-ui-doc-enable nil
        lsp-signature-auto-activate nil
        lsp-enable-symbol-highlighting nil
        lsp-ui-sideline-enable nil
        lsp-modeline-code-actions-enable nil
        lsp-headerline-breadcrumb-enable nil
        lsp-len-enable nil))
;; Shrink minibuffer
(after! vertico
  (setq vertico-count 8))
;; Eliminate gaps between windows
(after! frame
  (remove-hook 'doom-init-ui-hook #'window-divider-mode))
;; Reduce size of the popup terminal
(set-popup-rule! "*doom:vterm-popup:*" :size 0.25 :vslot -4 :select t :quit nil :ttl 0)
;; Show matching parenthesis on the minibuffer
(after! smartparens (show-smartparens-global-mode t))
;; Modify the splash screen actions
(setq +doom-dashboard-menu-sections
      '(("Reload last session"
         :icon (all-the-icons-octicon "history" :face 'doom-dashboard-menu-title)
         :when (cond ((modulep! :ui workspaces)
                      (file-exists-p (expand-file-name persp-auto-save-fname persp-save-dir)))
                     ((require 'desktop nil t)
                      (file-exists-p (desktop-full-file-name))))
         :face (:inherit (doom-dashboard-menu-title bold))
         :action doom/quickload-session)
        ("Open org-agenda"
         :icon (all-the-icons-octicon "calendar" :face 'doom-dashboard-menu-title)
         :when (fboundp 'org-agenda)
         :action org-agenda)
        ("Recently opened files"
         :icon (all-the-icons-octicon "file-text" :face 'doom-dashboard-menu-title)
         :action recentf-open-files)
        ("Open email client"
         :icon (all-the-icons-octicon "mail" :face 'doom-dashboard-menu-title)
         :when (featurep! :email mu4e)
         :face (:inherit (doom-dashboard-menu-title bold))
         :action =mu4e)
        ("RSS reader"
         :icon (all-the-icons-octicon "rss" :face 'doom-dashboard-menu-title)
         :when (featurep! :app rss)
         :face (:inherit (doom-dashboard-menu-title bold))
         :action =rss)
        ("Read saved articles"
         :icon (all-the-icons-octicon "book" :face 'doom-dashboard-menu-title)
         :face (:inherit (doom-dashboard-menu-title bold))
         :action =wallabag)))
;; Restore previous session without confirmation
(defadvice! restore-no-confirm-session-quickload-a ()
   :override #'doom/quickload-session
   (message "Restoring session...")
   (doom-load-session)
   (message "Session restored. Welcome back."))
;; Decrease size of icons in the modeline
(setq all-the-icons-scale-factor 1.0)
;; Increase size of the PlatformIO compilation buffer
(set-popup-rule! "*platformio*" :size 0.25 :vslot -4 :select t :quit t :ttl 0)
